#==============================================================================
# package      DE_EPAGES::GoogleSmartShopping::UI::Shop
# state        private
#==============================================================================
package DE_EPAGES::GoogleSmartShopping::UI::Shop;
use base DE_EPAGES::Presentation::UI::Object;

use DE_EPAGES::Core::API::Log qw ( LogError );
use DE_EPAGES::Core::API::PerlTools qw ( isValid );
use DE_EPAGES::GoogleSmartShopping::API::GoogleAdsApiService;
use DE_EPAGES::GoogleSmartShopping::API::GoogleApiService;
use DE_EPAGES::GoogleSmartShopping::API::GooglePortalService;
use DE_EPAGES::GoogleSmartShopping::API::GoogleSmartShoppingUrlBuilder;
use DE_EPAGES::GoogleSmartShopping::API::Tools qw ( LogGoogleSmartShopping GetErrorCode );
use DE_EPAGES::Core::API::Error qw ( ExistsError GetError Error );
use DE_EPAGES::Shop::API::Url qw ( BuildShopUrl );
use DE_EPAGES::GoogleSmartShopping::API::Constants qw (
    ACCOUNT_ISSUE_TRANSLATION_KEYS
    ERROR_ACCEPT_TERMS
    ERROR_FORM_SUBMIT
    ERROR_MISSING_API_SCOPES
    ERROR_TOO_MANY_SUBACCOUNTS
    ERROR_UNKNOWN
    GOOGLE_SMART_SHOPPING_APP_SCOPES
    ERROR_TOKEN_EXPIRED_OR_REVOKED
);

use strict;

use constant OAUTH_ONBOARDING => 0;
use constant ACCOUNT_CONNECTION_GET_EMAIL => 1;
use constant MERCHANT_CENTER_ACCOUNT_CONNECTED => 2;
use constant GOOGLE_ADS_ACCOUNT_CONNECTED => 3;
use constant IGNORED_ACCOUNT_ISSUES => [ qw (
    home_page_issue
    merchant_quality_low
    pending_phone_verification
) ];
use constant IGNORED_ACCOUNT_ISSUES_WHEN_NO_ADS_ACCOUNT => [ qw (
    missing_ad_words_link
) ];

use constant PHONE_VERIFICATION_ACCOUNT_ISSUE => 'pending_phone_verification';

#==============================================================================
# Event handlers
#==============================================================================
sub UnityViewGoogleSmartShopping {
    my $self = shift;
    my ($Servlet) = @_;

    my $Shop = $Servlet->object();
    my $aSetupStages = $self->_reckonSetupStages($Shop);

    if (OAUTH_ONBOARDING ~~ $aSetupStages) {
        $Servlet->vars('GoogleOAuthConnect', $self->_buildConnectOAuthURL($Servlet, $Shop));
        return $self->SUPER::View($Servlet);
    }

    my $GoogleSmartShoppingConfig = $Shop->get('GoogleSmartShoppingConfig');
    my $hGoogleSmartShoppingConfig = $GoogleSmartShoppingConfig->get([
        'ChecklistContactInfo',
        'ChecklistReturnPolicy',
        'ChecklistBillingTOC',
        'ChecklistGoogleAdsPolicies',
        'ChecklistCookieNotice',
        'MerchantCenterAccountName',
        'MerchantCenterAccountID',
        'GoogleAdsAccountID',
        'GoogleAdsAccountName',
        'IsAdsAccountSubAccount',
        'AreMerchantCenterAndAdsLinked'
    ]);

    $Servlet->vars('GoogleSmartShoppingConfig', $hGoogleSmartShoppingConfig);
    $Servlet->vars('GoogleMerchantCenterRequirementsMet', $GoogleSmartShoppingConfig->areGoogleMerchantCenterRequirementsMet());

    if (ACCOUNT_CONNECTION_GET_EMAIL ~~ $aSetupStages) {
        my $hEmailResult = $self->_getConnectedAccountEmail($Shop, $GoogleSmartShoppingConfig);
        if (defined($hEmailResult) && isValid($hEmailResult->{'email'})) {
            $Servlet->vars('ConnectedGoogleAccountEmail', $hEmailResult->{'email'});
            if ($hEmailResult->{'is_token_expired'}) {
                $Servlet->vars('TokenExpiredOrRevoked', 1);
                $Servlet->vars('GoogleOAuthConnect', $self->_buildConnectOAuthURL($Servlet, $Shop));
                return $self->SUPER::View($Servlet);
            }
        } else {
            $Servlet->vars('ConnectGoogleAccountError', 1);
            return $self->SUPER::View($Servlet);
        }
    }

    if (MERCHANT_CENTER_ACCOUNT_CONNECTED  ~~ $aSetupStages) {
        $Servlet->vars('MerchantCenterAccountConnected', 1);
        my $hMerchantCenterAccountStatuses = $self->_getMerchantCenterAccountStatuses($Shop);
        if(defined($hMerchantCenterAccountStatuses)) {
            $Servlet->vars('MerchantCenterAccountStatuses', $hMerchantCenterAccountStatuses);
        }

        my $GoogleSmartShoppingPortalConfig = $self->_getPortalConfig($Shop);

        unless (defined $GoogleSmartShoppingPortalConfig) {
            $GoogleSmartShoppingPortalConfig = $self->_createPortalConfig($Servlet, $Shop);
        }
        $Servlet->vars('PortalConfigID', $GoogleSmartShoppingPortalConfig->id());
    } else {
        my $MerchantCenterAccounts = $self->_getMerchantCenterAccounts($Shop, $GoogleSmartShoppingConfig);
        unless (defined($MerchantCenterAccounts)) {
            $Servlet->vars('MerchantCenterAccountsError', 1);
            return $self->SUPER::View($Servlet);
        }
        $Servlet->vars('MerchantCenterAccounts', $MerchantCenterAccounts);
    }

    if (GOOGLE_ADS_ACCOUNT_CONNECTED ~~ $aSetupStages) {

        $Servlet->vars('IsAdsAccountBillingApproved', $self->_isAdsAccountBillingApproved($Shop));
    } else {

        my $GoogleAdsAccounts = $self->_getGoogleAdsAccounts($Shop, $GoogleSmartShoppingConfig);
        unless (defined($GoogleAdsAccounts)) {
            $Servlet->vars('GoogleAdsAccountsListError', 1);
            return $self->SUPER::View($Servlet);
        }
        $Servlet->vars('GoogleAdsAccounts', $GoogleAdsAccounts);
    }

    return $self->SUPER::View($Servlet);
}

#----------------------------------------------------------------------------------------
sub _reckonSetupStages {
    my $self = shift;
    my ($Shop) = @_;

    return [ OAUTH_ONBOARDING ] unless ($Shop->existsChild('GoogleSmartShoppingConfig'));
    my $GoogleSmartShoppingConfig = $Shop->get('GoogleSmartShoppingConfig');
    return [ OAUTH_ONBOARDING ] unless ($GoogleSmartShoppingConfig->get('AccessToken'));
    return [ OAUTH_ONBOARDING ] unless ($GoogleSmartShoppingConfig->get('RefreshToken'));

    my $aSecondStageMultiStep = [ ACCOUNT_CONNECTION_GET_EMAIL ];

    if (isValid($GoogleSmartShoppingConfig->get('MerchantCenterAccountID'))) {
        push(@{$aSecondStageMultiStep}, MERCHANT_CENTER_ACCOUNT_CONNECTED);
    }

    if ($self->_isGoogleAdsAccountConnected($GoogleSmartShoppingConfig)) {
        push(@{$aSecondStageMultiStep}, GOOGLE_ADS_ACCOUNT_CONNECTED);
    }

    return $aSecondStageMultiStep;
}

#========================================================================================
# function     GoogleOAuthCallback
# state        public
#----------------------------------------------------------------------------------------
# syntax       GoogleOAuthCall( $Servlet );
#----------------------------------------------------------------------------------------
# description  ChangeAction callback handler for Google OAuth flow. After the redirect
#               from Google (via ePages Google OAuth app), we expect to be executing this
#               ChangeAction at the backend of a pop-up window.
#               If the correct parameters are passed, the method will exchange an OAuth
#               code for an AccessToken and a RefreshToken. These are then saved in the
#               GoogleSmartShoppingConfig object.
#----------------------------------------------------------------------------------------
# input        $Servlet | Presentation Object | object
#               $Servlet->form()->queryValues() contains the ?code? and ?scope?
#               parameters passed to us by the redirect from Google.
#----------------------------------------------------------------------------------------
# output       $Servlet->writeContentOutput() is used by this method to generate HTML
#               with embedded JavaScript that executes a payload to either reload the
#               parent window, or display an error in the parent window.
#========================================================================================
sub GoogleOAuthCallback {
    my $self = shift;
    my ($Servlet) = @_;

    my $Shop = $Servlet->object();
    my $Form = $Servlet->form();
    my $hQueryParameters = $Form->queryValues();

    my $ValidationError = $self->_validateOAuthCallbackParams($hQueryParameters);
    if ($ValidationError) {
        LogGoogleSmartShopping("Google OAuth validation error ($ValidationError): ", $hQueryParameters);
        $Servlet->vars('GoogleOAuthCallbackError', $ValidationError);
        return $self->SUPER::View($Servlet);
    }

    my $Service = DE_EPAGES::GoogleSmartShopping::API::GoogleApiService->new({Shop => $Shop});
    eval {
        $Service->getAndSaveOAuthTokens($hQueryParameters->{'code'}->[0]);
    };
    if (ExistsError()) {
        my $Error = GetError();
        my $ErrorCode = GetErrorCode($Error);
        LogGoogleSmartShopping("Google OAuth service error ($ErrorCode): ", $Error->message());
        $Servlet->vars('GoogleOAuthCallbackError', $ErrorCode);
    }

    return $self->SUPER::View($Servlet);
}

#----------------------------------------------------------------------------------------
sub _validateOAuthCallbackParams {
    my $self = shift;
    my ($hQueryParameters) = @_;

    return $hQueryParameters->{'error'}->[0] if ($hQueryParameters->{'error'});
    return ERROR_UNKNOWN unless ($hQueryParameters->{'code'} && $hQueryParameters->{'scope'});
    foreach my $RequiredScope (GOOGLE_SMART_SHOPPING_APP_SCOPES) {
        return ERROR_MISSING_API_SCOPES unless grep { $_ eq $RequiredScope } split ' ', $hQueryParameters->{'scope'}->[0];
    }
}

#========================================================================================
# function     CreateMerchantCenterAccount
# state        public
#----------------------------------------------------------------------------------------
# syntax       CreateMerchantCenterAccount( $Servlet );
#----------------------------------------------------------------------------------------
# description  Create a new SubAccount on ePages MerchantCenterAccount
#----------------------------------------------------------------------------------------
# input        $Servlet | Presentation Object | object
# return       $Account | Account details for created MerchantCenterAccount | object
#========================================================================================
sub CreateMerchantCenterAccount {
    my $self = shift;
    my ($Servlet) = @_;

    my $Shop = $Servlet->object();
    my $Form = $Servlet->form();
    my $hFormValues = $Form->formValues();

    my $GoogleSmartShoppingConfig = $Shop->get('GoogleSmartShoppingConfig');
    Error('Undefined configuration for GoogleSmartShopping') unless defined $GoogleSmartShoppingConfig;

    my $FormErrorReason = $self->_validateCreateSubAccountRequest($hFormValues, $GoogleSmartShoppingConfig);

    if ($FormErrorReason) {
        $Form->executeFormError({
            'Name'   => 'MerchantCenterAccountCreationFailed',
            'Reason' => $FormErrorReason
        });
    }

    my $GoogleApiService = DE_EPAGES::GoogleSmartShopping::API::GoogleApiService->new({Shop => $Shop});

    my $NewMerchantCenterSubAccount;
    eval {
        $NewMerchantCenterSubAccount = $GoogleApiService->createMerchantCenterSubAccount(
            $hFormValues->{'HasAdultContent'}->[0]
        );
    };
    if (ExistsError()) {
        my $Error = GetError();
        LogGoogleSmartShopping('Account creation failed', {
            'Error' => $Error->message()
        });
        my $ErrorCode = GetErrorCode($Error);
        if ($ErrorCode eq ERROR_TOO_MANY_SUBACCOUNTS) {
            $self->_logError($Error);
        }
        $Form->executeFormError({
            'Name'   => 'MerchantCenterAccountCreationFailed',
            'Reason' => $ErrorCode
        });
    }

    $Servlet->vars('ResponseContent', $NewMerchantCenterSubAccount);
}

#========================================================================================
# function     CreateGoogleAdsAccount
# state        public
#----------------------------------------------------------------------------------------
# syntax       CreateGoogleAdsAccount( $Servlet );
#----------------------------------------------------------------------------------------
# description  Create a new SubAccount on ePages GoogleAdsAccount
#----------------------------------------------------------------------------------------
# input        $Servlet | Presentation Object | object
# return       $Account | Account details for created GoogleAdsAccount | object
#========================================================================================
sub CreateGoogleAdsAccount {
    my $self = shift;
    my ($Servlet) = @_;

    my $Shop = $Servlet->object();
    my $Form = $Servlet->form();

    my $GoogleSmartShoppingConfig = $Shop->get('GoogleSmartShoppingConfig');
    Error('Undefined configuration for GoogleSmartShopping') unless defined $GoogleSmartShoppingConfig;

    my $FormErrorReason = $self->_validateCreateGoogleAdsSubAccountRequest($GoogleSmartShoppingConfig);
    if (defined $FormErrorReason) {
        $Form->executeFormError({
            'Name'   => 'AdsAccountCreationFailed',
            'Reason' => $FormErrorReason
        });
    }

    my $GoogleAdsApiService = DE_EPAGES::GoogleSmartShopping::API::GoogleAdsApiService->new({Shop => $Shop});

    eval {
        $GoogleAdsApiService->createGoogleAdsSubAccount();
    };

    if (ExistsError()) {
        my $Error = GetError();
        LogGoogleSmartShopping('Account creation failed', {
            'Error' => $Error->message(),
        });

        $Form->executeFormError({
            'Name'   => 'AdsAccountCreationFailed',
            'Reason' => GetErrorCode($Error)
        });
    }

    eval {
        $GoogleAdsApiService->createAndEnableGoogleAdsLink();
    };

    if (ExistsError()) {
        my $Error = GetError();
        LogGoogleSmartShopping('Account linking failed', {
            'Error' => $Error->message(),
        });

        $Form->executeFormError({
            'Name'   => 'ErrorMerchantCenterAdsLinking',
            'Reason' => GetErrorCode($Error)
        });
    }
}

#========================================================================================
# function     DisconnectPersonalAccount
# state        public
#----------------------------------------------------------------------------------------
# syntax       DisconnectPersonalAccount( $Servlet );
#----------------------------------------------------------------------------------------
# description  Disconnects the Google personal account and MerchantCenter account
#----------------------------------------------------------------------------------------
# input        $Servlet | presentation object | object
#========================================================================================
sub DisconnectPersonalAccount {
    my $self = shift;
    my ($Servlet) = @_;

    my $Shop = $Servlet->object();
    my $Form = $Servlet->form();
    my $GoogleSmartShoppingConfig = $Shop->get('GoogleSmartShoppingConfig');
    return unless defined($GoogleSmartShoppingConfig);

    if (isValid($GoogleSmartShoppingConfig->get('GoogleAdsAccountID'))) {
        $self->_disconnectAdsAccountAndCampaign($Shop, $Form, $GoogleSmartShoppingConfig);
    }

    if (isValid($GoogleSmartShoppingConfig->get('MerchantCenterAccountID'))) {
        $self->_disconnectMerchantCenterAccount($Shop, $Form, $GoogleSmartShoppingConfig);
    }

    my $GoogleApiService = DE_EPAGES::GoogleSmartShopping::API::GoogleApiService->new({Shop => $Shop});
    eval {
        my $AccessToken = $GoogleSmartShoppingConfig->get('AccessToken');
        $GoogleApiService->revokeAccessToken($AccessToken);
    };
    if (ExistsError()) {
        my $Error = GetError();
        LogGoogleSmartShopping('DisconnectPersonalAccount failed', {
            'Error' => $Error->message(),
        });

        $Form->addFormError({
            'Name' => 'DisconnectPersonalAccount',
            'Reason' => GetErrorCode($Error)
        });
    }

    $GoogleSmartShoppingConfig->delete();
    LogGoogleSmartShopping("GoogleSmartShoppingConfig deleted for Shop ", $Shop->id() );

    $self->_deleteGooglePortalConfig($Shop);

    $Form->executeFormError();
    return;
}

sub _deleteGooglePortalConfig {
    my $self = shift;
    my ($Shop) = @_;
    my $GooglePortalService = DE_EPAGES::GoogleSmartShopping::API::GooglePortalService->new({Shop => $Shop});

    $GooglePortalService->deleteSmartShoppingPortalConfig();
    return;
}

#========================================================================================
# function     DisconnectMerchantCenterAccount
# state        public
#----------------------------------------------------------------------------------------
# syntax       DisconnectMerchantCenterAccount( $Servlet );
#----------------------------------------------------------------------------------------
# description  Disconnects the MerchantCenterAccount
#----------------------------------------------------------------------------------------
# input        $Servlet | presentation object | object
#========================================================================================
sub DisconnectMerchantCenterAccount {
    my $self = shift;
    my ($Servlet) = @_;

    my $Shop = $Servlet->object();
    my $Form = $Servlet->form();
    my $GoogleSmartShoppingConfig = $Shop->get('GoogleSmartShoppingConfig');
    return unless defined($GoogleSmartShoppingConfig);

    $self->_disconnectMerchantCenterAccount($Shop, $Form, $GoogleSmartShoppingConfig);

    $self->_deleteGooglePortalConfig($Shop);

    $Form->executeFormError();
    return;
}

sub LinkMerchantCenterAndAds {
    my $self = shift;
    my ($Servlet) = @_;

    my $Shop = $Servlet->object();
    my $Form = $Servlet->form();
    my $Service = DE_EPAGES::GoogleSmartShopping::API::GoogleAdsApiService->new({ Shop => $Shop });
    eval {
        $Service->createAndEnableGoogleAdsLink();
    };
    if (ExistsError()) {
        my $Error = GetError();
        LogGoogleSmartShopping('linking merchant center and Ads account failed', {
            'Error' => $Error->message(),
        });

        $Form->executeFormError({
            'Name' => 'MerchantCenterAdsLinkingFail',
            'Reason' => GetErrorCode($Error)
        });
    }
}

#========================================================================================
# function     UnitySFViewGoogleSmartShoppingHTML
# state        public
#----------------------------------------------------------------------------------------
# syntax       UnitySFViewGoogleSmartShoppingHTML( $Servlet );
#----------------------------------------------------------------------------------------
# description  Hook function for site verification
#               Generates HTML response with google-site-verification token
#----------------------------------------------------------------------------------------
# input        $Servlet | presentation object | object
#========================================================================================
sub UnitySFViewGoogleSmartShoppingHTML {
    my $self = shift;
    my ($Servlet) = @_;
    my $Shop = $Servlet->object->get('Site');
    my $Response = $Servlet->response;
    $Response->header( 'Content-Type' => 'text/html' );

    return unless ($Shop->existsChild('GoogleSmartShoppingConfig'));
    my $GoogleSmartShoppingConfig = $Shop->get('GoogleSmartShoppingConfig');

    my $GSSSiteVerificationToken= $GoogleSmartShoppingConfig->get('SiteVerificationToken');

    my $RequestURI;
    if ($self->_getServerVariables($Servlet, 'QUERY_STRING') =~ /^URI=([^&]+)/) {
        $RequestURI = $1;
    }
    if($GSSSiteVerificationToken && $GSSSiteVerificationToken eq $RequestURI) {
        # display html file for Google Smart Shopping
        my $Content = 'google-site-verification: ' . $GSSSiteVerificationToken;
        $Response->content($Content);
    } else {
        # display 404 error
        $Response->content('No google*.html available.');
        $Response->code('404');
    }

    $Servlet->vars('ExitAfterEvent', 1);

    return;
}

#========================================================================================
# function     DisconnectGoogleAdsAccount
# state        public
#----------------------------------------------------------------------------------------
# syntax       DisconnectGoogleAdsAccount( $Servlet );
#----------------------------------------------------------------------------------------
# description  Disconnects the GoogleAdsAccount
#----------------------------------------------------------------------------------------
# input        $Servlet | presentation object | object
#========================================================================================
sub DisconnectGoogleAdsAccount {
    my $self = shift;
    my ($Servlet) = @_;

    my $Shop = $Servlet->object();
    my $Form = $Servlet->form();

    my $GoogleSmartShoppingConfig = $Shop->get('GoogleSmartShoppingConfig');
    return unless defined($GoogleSmartShoppingConfig);

    $self->_disconnectAdsAccountAndCampaign($Shop, $Form, $GoogleSmartShoppingConfig);

    $Form->executeFormError();
    return;
}

#----------------------------------------------------------------------------------------
sub _disconnectMerchantCenterAccount {
    my $self = shift;
    my ($Shop, $Form, $GoogleSmartShoppingConfig) = @_;

    if ($GoogleSmartShoppingConfig->get('IsMerchantCenterSubAccount')) {

        my $GoogleApiService = DE_EPAGES::GoogleSmartShopping::API::GoogleApiService->new({Shop => $Shop});
        eval {
            $GoogleApiService->deleteMerchantCenterSubAccount($GoogleSmartShoppingConfig->get('MerchantCenterAccountID'));
        };
        if (ExistsError()) {
            my $Error = GetError();

            LogGoogleSmartShopping('DisconnectMerchantCenterAccount failed', {
                'Error' => $Error->message(),
            });

            $Form->addFormError({
                'Name' => 'DisconnectMerchantCenterAccount',
                'Reason' => GetErrorCode($Error)
            });
        }
    }

    $GoogleSmartShoppingConfig->deleteMerchantCenterAccount();

    LogGoogleSmartShopping("GoogleSmartShoppingConfig MerchantCenterAccount deleted", {});
    return;
}
#----------------------------------------------------------------------------------------
sub _disconnectAdsAccountAndCampaign {
    my $self = shift;
    my ($Shop, $Form, $GoogleSmartShoppingConfig) = @_;

    my $GoogleAdsApiService = DE_EPAGES::GoogleSmartShopping::API::GoogleAdsApiService->new({Shop => $Shop});

    my $AdsCampaignId = $GoogleSmartShoppingConfig->get('AdsCampaignId');
    if (isValid($AdsCampaignId)) {
        eval {
            $GoogleAdsApiService->deleteAdsCampaign($AdsCampaignId);
        };
        if (ExistsError()) {
            my $Error = GetError();
            LogGoogleSmartShopping('deleteAdsCampaign failed', { 'Error' => $Error->message() });
            $Form->addFormError({
                'Name'   => 'CampaignDeletionFailed',
                'Reason' => GetErrorCode($Error)
            });
        }
        $GoogleSmartShoppingConfig->deleteAdsCampaign();
        LogGoogleSmartShopping("GoogleSmartShoppingConfig AdsCampaign deleted", {});
    }

    eval {
        $GoogleAdsApiService->unlinkGoogleAdsAccount();
    };
    if (ExistsError()) {
        my $Error = GetError();
        LogGoogleSmartShopping('Unlink Ads Account failed', { 'Error' => $Error->message() });
        $Form->addFormError({
            'Name'   => 'AdsAccountUnlinkFailed',
            'Reason' => GetErrorCode($Error)
        });
    }

    $GoogleSmartShoppingConfig->deleteAdsAccount();
    LogGoogleSmartShopping("GoogleSmartShoppingConfig AdsAccount deleted", {});

    return;
}
#----------------------------------------------------------------------------------------
sub _validateCreateSubAccountRequest {
    my $self = shift;
    my ($hFormValues, $GoogleSmartShoppingConfig) = @_;

    return ERROR_FORM_SUBMIT unless (isValid($hFormValues->{'HasAdultContent'}) && isValid($hFormValues->{'HasAdultContent'}->[0]));
    return ERROR_ACCEPT_TERMS unless (isValid($hFormValues->{'newMerchantAccountAcceptTerms'}) && $hFormValues->{'newMerchantAccountAcceptTerms'}->[0]);
    return ERROR_ACCEPT_TERMS unless ($GoogleSmartShoppingConfig->areGoogleMerchantCenterRequirementsMet());

    return;
}
#----------------------------------------------------------------------------------------
sub _buildConnectOAuthURL {
    my $self = shift;
    my ( $Servlet, $Shop ) = @_;

    my $hParams = {
        'ViewAction' => 'UnityMBO-GoogleOAuthCallback',
        'ObjectID'   => $Shop->id(),
    };
    my $hOptions = {
        'Type' => 'admin',
        'AddToken' => 1,
        'UseSSL' => 1,
    };
    my $CallbackUrl = BuildShopUrl($Shop, $hParams, $hOptions, $Servlet);
    LogGoogleSmartShopping('Callback URL is:', $CallbackUrl);
    my $GoogleUrlBuilder = DE_EPAGES::GoogleSmartShopping::API::GoogleSmartShoppingUrlBuilder->new();

    return $GoogleUrlBuilder->buildConnectOAuthUrl($CallbackUrl);
}

#----------------------------------------------------------------------------------------
sub _getConnectedAccountEmail {
    my $self = shift;
    my ($Shop, $GoogleSmartShoppingConfig) = @_;

    my $Service = DE_EPAGES::GoogleSmartShopping::API::GoogleApiService->new({Shop => $Shop});
    my $hResponse = {};
    eval {
        $hResponse->{'email'} = $Service->getOAuthEmail();
    };
    if (ExistsError()) {
        my $Error = GetError();
        LogGoogleSmartShopping("Google OpenID userinfo lookup failed: ", $Error->message());
        $hResponse->{'email'} = $GoogleSmartShoppingConfig->get('AccountEmail');
        my $ErrorCode = GetErrorCode($Error);
        if ($ErrorCode eq ERROR_TOKEN_EXPIRED_OR_REVOKED) {
            $hResponse->{'is_token_expired'} = 1;
        }
    }
    return $hResponse;
}

sub _getMerchantCenterAccounts {
    my $self = shift;
    my ($Shop) = @_;

    my $Service = DE_EPAGES::GoogleSmartShopping::API::GoogleApiService->new({Shop => $Shop});
    my $aResponse;
    eval {
        $aResponse = $Service->getMerchantCenterAccounts();
    };
    if (ExistsError()) {
        my $Error = GetError();
        LogGoogleSmartShopping("Service->getMerchantCenterAccounts() failed - ", $Error->message());
    } else {
        LogGoogleSmartShopping("Service->getMerchantCenterAccounts() response - ", $aResponse);
    }
    return $aResponse;
}

sub _isAdsAccountBillingApproved {
    my $self = shift;
    my ($Shop) = @_;

    my $Service = DE_EPAGES::GoogleSmartShopping::API::GoogleAdsApiService->new({Shop => $Shop});
    my $isAdsBillingApproved;
    eval {
        $isAdsBillingApproved = $Service->isAdsAccountBillingApproved();
    };
    if (ExistsError()) {
        my $Error = GetError();
        LogGoogleSmartShopping('_isAdsAccountBillingApproved failed - ', $Error->message());
        return 0;
    }

    return $isAdsBillingApproved;
}

#----------------------------------------------------------------------------------------
sub _getMerchantCenterAccountStatuses {
    my $self = shift;
    my ($Shop) = @_;

    my $GoogleSmartShoppingConfig = $Shop->get('GoogleSmartShoppingConfig');
    return unless defined $GoogleSmartShoppingConfig;

    my $Service = DE_EPAGES::GoogleSmartShopping::API::GoogleApiService->new({Shop => $Shop});

    my $hResult;
    eval {
        $hResult = $Service->getMerchantCenterAccountStatuses();
    };
    if (ExistsError()) {
        my $Error = GetError();
        LogGoogleSmartShopping("Google accountLevelIssues request failed: ", $Error->message());
    }


    my $hFilteredIssues = $self->_removeIgnoredIssues($hResult->{'accountLevelIssues'}, $GoogleSmartShoppingConfig);
    foreach my $AccountIssue (@{ $hFilteredIssues }) {
        if (defined(ACCOUNT_ISSUE_TRANSLATION_KEYS->{ $AccountIssue->{'id'} })) {
            $AccountIssue->{'translationKey'} = ACCOUNT_ISSUE_TRANSLATION_KEYS->{ $AccountIssue->{'id'} };
        }
    }

    my $hMerchantCenterAccountStatuses = {
        AccountLevelIssues => $hFilteredIssues,
        IsWebsiteClaimed   => ($hResult->{'websiteClaimed'}) ? 1 : 0,
        IsPhoneVerified    => ($self->_isPendingPhoneVerification($hResult->{'accountLevelIssues'})) ? 0 : 1
    };

    return $hMerchantCenterAccountStatuses;
}

sub _isGoogleAdsAccountConnected {
    my $self = shift;
    my ($GoogleSmartShoppingConfig) = @_;

    return isValid($GoogleSmartShoppingConfig->get('GoogleAdsAccountID'));
}

sub _validateCreateGoogleAdsSubAccountRequest {
    my $self = shift;
    my ($GoogleSmartShoppingConfig) = @_;
    return ERROR_ACCEPT_TERMS unless ($GoogleSmartShoppingConfig->areGoogleMerchantCenterRequirementsMet());
    return ERROR_FORM_SUBMIT unless (isValid($GoogleSmartShoppingConfig->get('MerchantCenterAccountID')));
    return ERROR_FORM_SUBMIT if $self->_isGoogleAdsAccountConnected($GoogleSmartShoppingConfig);
    return;
}

sub _getGoogleAdsAccounts {
    my $self = shift;
    my ($Shop) = @_;

    my $Service = DE_EPAGES::GoogleSmartShopping::API::GoogleAdsApiService->new({Shop => $Shop});
    my $AdsAccounts;
    eval {
        $AdsAccounts = $Service->getGoogleAdsAccounts();
        LogGoogleSmartShopping('GoogleAdsAccounts:', $AdsAccounts);
    };
    if (ExistsError()) {
        my $Error = GetError();
        LogGoogleSmartShopping('Error getting GoogleAdsAccounts: ', $Error->message());
    }
    return $AdsAccounts;
}

sub _removeIgnoredIssues {
    my $self = shift;
    my ($AccountLevelIssues, $GoogleSmartShoppingConfig) = @_;

    $AccountLevelIssues =  [
        grep { ! ($_->{'id'} ~~ IGNORED_ACCOUNT_ISSUES) } @$AccountLevelIssues
    ];

    if(! $self->_isGoogleAdsAccountConnected($GoogleSmartShoppingConfig)) {
        $AccountLevelIssues =  [
            grep { ! ($_->{'id'} ~~ IGNORED_ACCOUNT_ISSUES_WHEN_NO_ADS_ACCOUNT) } @$AccountLevelIssues
        ];
    }

    return $AccountLevelIssues;
}

sub _getServerVariables {
    my $self = shift;
    my ($Servlet, $VariableName) = @_;

    return $Servlet->serverVariables($VariableName);
}

sub _isPendingPhoneVerification {
    my $self = shift;
    my ($aAccountLevelIssues) = @_;

    my @aIssueIds = map { $_->{'id'} } @$aAccountLevelIssues;

    return PHONE_VERIFICATION_ACCOUNT_ISSUE ~~ \@aIssueIds;
}

sub _logError {
    my $self = shift;
    my ($Error) = @_;

    LogError($Error);
}

sub _getPortalConfig {
    my $self = shift;
    my ($Shop) = @_;
    my $GooglePortalService = DE_EPAGES::GoogleSmartShopping::API::GooglePortalService->new({ Shop => $Shop });
    return $GooglePortalService->getSmartShoppingPortalConfig();
}

sub _createPortalConfig {
    my $self = shift;
    my ($Servlet, $Shop) = @_;
    my $GooglePortalService = DE_EPAGES::GoogleSmartShopping::API::GooglePortalService->new({ Shop => $Shop });
    my $hPortalConfigCreationParams = $GooglePortalService->getPortalConfigCreationParams();
    unless (defined $hPortalConfigCreationParams) {
        LogGoogleSmartShopping("PortalConfig could not be created", $hPortalConfigCreationParams);
        return;
    }
    LogGoogleSmartShopping("PortalConfig created with Alias", $hPortalConfigCreationParams->{'PortalSite'}->alias());
    return $self->NewObject($Servlet, $hPortalConfigCreationParams->{'Folder'}, $hPortalConfigCreationParams->{'Class'}, {
        'Alias'      => $hPortalConfigCreationParams->{'PortalSite'}->alias(),
        'PortalSite' => $hPortalConfigCreationParams->{'PortalSite'},
    });

}

1;
