#========================================================================================
# package      DE_EPAGES::GoogleSmartShopping::t::unit::ProductFeed
# base         DE_EPAGES::Core::API::Test
# state        public
#----------------------------------------------------------------------------------------
# description  Unit test for the GoogleSmartShopping ProductFeed tab
#========================================================================================
package DE_EPAGES::GoogleSmartShopping::t::unit::ProductFeed;
use base DE_EPAGES::Core::API::Test;

use DE_EPAGES::Test::Mock::MockifyEpagesObject;
use DE_EPAGES::Test::Mock::MockifyServlet;

use Test::Mockify;
use Test::Mockify::Matcher qw ( String HashRef Object );
use Test::Mockify::Sut;
use Test::Mockify::Verify qw ( WasCalled GetParametersFromMockifyCall );

use HTTP::Status qw (HTTP_OK );

use Test::More;

use strict;

use constant MERCHANT_CENTER_ID => 503220055;
use constant GOOGLE_PORTAL_ALIAS => 'SomeAlias_de_DE';
use constant DEFAULT_SHIPPING_METHOD_ID => '12345';
use constant DEFAULT_SHIPPING_METHOD_ALIAS => 'Express';


sub test {
    my $self = shift;

    $self->test_UnityViewGoogleSmartShoppingProductFeed_success_existingPortalConfig();
    $self->test_UnityViewGoogleSmartShoppingProductFeed_success_newPortalConfig();
    $self->test_UnityViewGoogleSmartShoppingProductFeed_fail_NoShopAddress();
    $self->test_UnityViewGoogleSmartShoppingProductFeed_fail_NoSmartshoppingConfig();
    $self->test_UnityViewGoogleSmartShoppingProductFeed_fail_MerchantCenterRequirementsNotMet();
    $self->test_UnityViewGoogleSmartShoppingProductFeed_Success_ProductsStatistics();
    $self->test_ChangePortalShippingMethod_Success_without_Products();
    $self->test_ChangePortalShippingMethod_Success_with_Products();
    $self->test_ChangePortalShippingMethod_fail_no_PortalConfig();
    $self->test_UploadAllProducts();

    done_testing();
}

sub test_UnityViewGoogleSmartShoppingProductFeed_success_existingPortalConfig {
    my $self = shift;
    my $SubTestName = ( caller(0) )[3];

    my $hGoogleConfigParams = {
        MerchantCenterAccountID                => MERCHANT_CENTER_ID,
        AreGoogleMerchantCenterRequirementsMet => 1,
    };
    my $MockedGoogleConfig = $self->_createGoogleConfig($hGoogleConfigParams);

    my $hCountry = {'Code2' => 'DE'};
    my $MockedAddress = $self->_createAddress($hCountry);

    my $hShopParams = {
         MockedAddress      => $MockedAddress,
         MockedGoogleConfig => $MockedGoogleConfig,
    };
    my $MockedShop = $self->_createShop($hShopParams);

    my $MockedPortalConfig = $self->_createPortalConfig({
         MockedShop => $MockedShop,
    });

    my $hPostData = {};
    my $ServletForm = $self->_createServletForm($hPostData);

    my $MockedServlet = $self->_createServlet($MockedPortalConfig, $ServletForm);

    my $hSUTParams = {
        'ExistingGooglePortalService' => $self->_createPortalConfig(),
    };
    my $SUT = $self->_createSUT($hSUTParams);

    $SUT->UnityViewGoogleSmartShoppingProductFeed($MockedServlet);

    ok( WasCalled($SUT, '_areProductUploadRequirementsMet'), $SubTestName . ' - Verify that SUT "_areProductUploadRequirementsMet" method was called.' );

    is ($MockedServlet->vars('IsAllowedProductUpload'), 1, $SubTestName . ' - Verify that upload is flagged allowed.' );
}

sub test_UnityViewGoogleSmartShoppingProductFeed_success_newPortalConfig {
    my $self = shift;
    my $SubTestName = ( caller(0) )[3];

    my $hGoogleConfigParams = {
        MerchantCenterAccountID                => MERCHANT_CENTER_ID,
        AreGoogleMerchantCenterRequirementsMet => 1,
    };
    my $MockedGoogleConfig = $self->_createGoogleConfig($hGoogleConfigParams);

    my $hCountry = {'Code2' => 'DE'};
    my $MockedAddress = $self->_createAddress($hCountry);

    my $hShopParams = {
        MockedAddress      => $MockedAddress,
        MockedGoogleConfig => $MockedGoogleConfig,
    };
    my $MockedShop = $self->_createShop($hShopParams);

    my $MockedPortalConfig = $self->_createPortalConfig({
         MockedShop => $MockedShop,
    });

    my $MockedServlet = $self->_createServlet($MockedPortalConfig);
    my $SUT = $self->_createSUT();

    $SUT->UnityViewGoogleSmartShoppingProductFeed($MockedServlet);

    ok( WasCalled($SUT, '_areProductUploadRequirementsMet'), $SubTestName . ' - Verify that SUT "_areProductUploadRequirementsMet" method was called.' );

    is ($MockedServlet->vars('IsAllowedProductUpload'), 1, $SubTestName . ' - Verify that upload is flagged allowed.' );
}

sub test_UnityViewGoogleSmartShoppingProductFeed_fail_NoShopAddress {
    my $self = shift;
    my $SubTestName = ( caller(0) )[3];

    my $hGoogleConfigParams = {
        MerchantCenterAccountID                => MERCHANT_CENTER_ID,
        AreGoogleMerchantCenterRequirementsMet => 1,
    };
    my $MockedGoogleConfig = $self->_createGoogleConfig($hGoogleConfigParams);

    my $hCountry = undef;
    my $MockedAddress = $self->_createAddress($hCountry);
    my $hShopParams = {
        MockedAddress      => $MockedAddress,
        MockedGoogleConfig => $MockedGoogleConfig,
    };
    my $MockedShop = $self->_createShop($hShopParams);

    my $MockedPortalConfig = $self->_createPortalConfig({
         MockedShop => $MockedShop,
    });

    my $MockedServlet = $self->_createServlet($MockedPortalConfig);

    my $SUT = $self->_createSUT();

    $SUT->UnityViewGoogleSmartShoppingProductFeed($MockedServlet);

    ok( WasCalled($SUT, '_areProductUploadRequirementsMet'), $SubTestName . ' - Verify that SUT "_areProductUploadRequirementsMet" method was called.' );

    is ($MockedServlet->vars('IsAllowedProductUpload'), undef, $SubTestName . ' - Verify that upload is flagged forbidden.' );
    is ($MockedServlet->vars('ShippingMethodId'), undef, $SubTestName . ' - Verify that Servlet contains no Shipping Method.' );

}

sub test_UnityViewGoogleSmartShoppingProductFeed_fail_NoSmartshoppingConfig {
    my $self = shift;
    my $SubTestName = ( caller(0) )[3];

    my $hCountry = {'Code2' => 'DE'};
    my $MockedAddress = $self->_createAddress($hCountry);

    my $hShopParams = {
        MockedAddress      => $MockedAddress,
    };
    my $MockedShop = $self->_createShop($hShopParams);

    my $MockedPortalConfig = $self->_createPortalConfig({
         MockedShop => $MockedShop,
    });

    my $MockedServlet = $self->_createServlet($MockedPortalConfig);

    my $SUT = $self->_createSUT();

    $SUT->UnityViewGoogleSmartShoppingProductFeed($MockedServlet);

    ok( WasCalled($SUT, '_areProductUploadRequirementsMet'), $SubTestName . ' - Verify that SUT "_areProductUploadRequirementsMet" method was called.' );
    is ($MockedServlet->vars('IsAllowedProductUpload'), undef, $SubTestName . ' - Verify that upload is flagged forbidden.' );
    is ($MockedServlet->vars('ShippingMethodId'), undef, $SubTestName . ' - Verify that Servlet contains no Shipping Method.' );
}

sub test_UnityViewGoogleSmartShoppingProductFeed_fail_MerchantCenterRequirementsNotMet {
    my $self = shift;
    my $SubTestName = ( caller(0) )[3];

    my $hGoogleConfigParams = {
        MerchantCenterAccountID                => MERCHANT_CENTER_ID,
        AreGoogleMerchantCenterRequirementsMet => 0,
    };
    my $MockedGoogleConfig = $self->_createGoogleConfig($hGoogleConfigParams);

    my $hCountry = {'Code2' => 'DE'};
    my $MockedAddress = $self->_createAddress($hCountry);

    my $hShopParams = {
        MockedAddress      => $MockedAddress,
        MockedGoogleConfig => $MockedGoogleConfig,
    };
    my $MockedShop = $self->_createShop($hShopParams);

    my $MockedPortalConfig = $self->_createPortalConfig({
         MockedShop => $MockedShop,
    });

    my $MockedServlet = $self->_createServlet($MockedPortalConfig);

    my $SUT = $self->_createSUT();

    $SUT->UnityViewGoogleSmartShoppingProductFeed($MockedServlet);

    ok( WasCalled($SUT, '_areProductUploadRequirementsMet'), $SubTestName . ' - Verify that SUT "_areProductUploadRequirementsMet" method was called.' );

    is ($MockedServlet->vars('IsAllowedProductUpload'), undef, $SubTestName . ' - Verify that upload is flagged forbidden.' );
    is ($MockedServlet->vars('ShippingMethodId'), undef, $SubTestName . ' - Verify that Servlet contains no Shipping Method.' );
}

sub test_UnityViewGoogleSmartShoppingProductFeed_Success_ProductsStatistics {
    my $self = shift;
    my $SubTestName = ( caller(0) )[3];

    my $hGoogleConfigParams = {
        MerchantCenterAccountID                => MERCHANT_CENTER_ID,
        AreGoogleMerchantCenterRequirementsMet => 1,
    };
    my $MockedGoogleConfig = $self->_createGoogleConfig($hGoogleConfigParams);

    my $hCountry = {'Code2' => 'DE'};
    my $MockedAddress = $self->_createAddress($hCountry);

    my $hShopParams = {
        MockedAddress      => $MockedAddress,
        MockedGoogleConfig => $MockedGoogleConfig,
    };
    my $MockedShop = $self->_createShop($hShopParams);

    my $MockedPortalConfig = $self->_createPortalConfig({
         MockedShop => $MockedShop,
    });

    my $MockedServlet = $self->_createServlet($MockedPortalConfig);

    my $ExpectedProductsStatistics = {
        TooManyDisapproved  => 0,
        TooManyPending      => 0,
        NotEnoughActive     => 0,
        total               => 5,
        active              => 5,
        pending             => 0,
        disapproved         => 0
    };

    my $GoogleApiService = $self->_createGoogleApiService({
        RatedProductsStatistics => $ExpectedProductsStatistics
    });

    my $hParams = {
        'GoogleApiService'          => $GoogleApiService,
        'ExistingGooglePortalService' => $self->_createPortalConfig()
    };
    my $SUT = $self->_createSUT($hParams);

    $SUT->UnityViewGoogleSmartShoppingProductFeed($MockedServlet);

    ok(WasCalled($GoogleApiService, 'getRatedProductsStatistics'),
        $SubTestName . ' - Verify that service method _getProductStatistics was called.');
    is_deeply(
        $MockedServlet->vars('ProductStatuses'),
        $ExpectedProductsStatistics,
        "$SubTestName - Verify that ProductStatistics was correctly set in servlet."
    );
}

sub test_ChangePortalShippingMethod_Success_without_Products {
    my $self = shift;
    my $SubTestName = ( caller(0) )[3];


    my $MockedShop = $self->_createShop({});

    my $MockedPortalConfig = $self->_createPortalConfig({
         MockedShop => $MockedShop,
    });

    my $hFormValues = {
        selectedShippingMethodId => ['8198228'],
    };
    my $MockedServletForm  = $self->_createServletForm($hFormValues);
    my $MockedServletResponse = $self->_createServletResponse();
    my $MockedServlet = $self->_createServlet($MockedPortalConfig, $MockedServletForm, $MockedServletResponse);

    my $SUT = $self->_createSUT();

    $SUT->ChangePortalShippingMethod($MockedServlet);

    my $Code = GetParametersFromMockifyCall($MockedServletResponse, 'code')->[0];
    my $aParams = GetParametersFromMockifyCall($MockedServlet, 'params');
    my $hContent = GetParametersFromMockifyCall($MockedServlet, 'writeContentOutput')->[0];
    # actual tests
    is(
        $Code,
        HTTP_OK,
        $SubTestName . ' - Ensure Ensure HTTP response code was HTTP_OK'
    );
    is_deeply(
        $aParams,
        [ 'ContentType' => 'application/json' ],
        $SubTestName . ' - Ensure ContentType of application/json'
    );
    is_deeply(
        $hContent->{'ShippingMethodID'},
        '8198228',
        $SubTestName . ' - Ensure $Servlet->writeContentOutput() was called with updated ShippingMethodID'
    );
}

sub test_ChangePortalShippingMethod_Success_with_Products {
    my $self = shift;
    my $SubTestName = ( caller(0) )[3];


    my $MockedShop = $self->_createShop({  });

    my $MockedPortalConfig = $self->_createPortalConfig({
         MockedShop => $MockedShop,
         Products => ['12334', '87101'],
    });

    my $hFormValues = {
        selectedShippingMethodId => ['8198228'],
    };
    my $MockedServletForm  = $self->_createServletForm($hFormValues);
    my $MockedServletResponse = $self->_createServletResponse();
    my $MockedServlet = $self->_createServlet($MockedPortalConfig, $MockedServletForm, $MockedServletResponse);

    my $SUT = $self->_createSUT();

    $SUT->ChangePortalShippingMethod($MockedServlet);

    my $Code = GetParametersFromMockifyCall($MockedServletResponse, 'code')->[0];
    my $aParams = GetParametersFromMockifyCall($MockedServlet, 'params');
    my $hContent = GetParametersFromMockifyCall($MockedServlet, 'writeContentOutput')->[0];
    # actual tests
    is(
        $Code,
        HTTP_OK,
        $SubTestName . ' - Ensure Ensure HTTP response code was HTTP_OK'
    );
    is_deeply(
        $aParams,
        [ 'ContentType' => 'application/json' ],
        $SubTestName . ' - Ensure ContentType of application/json'
    );
    is_deeply(
        $hContent->{'ShippingMethodID'},
        DEFAULT_SHIPPING_METHOD_ID,
        $SubTestName . ' - Ensure $Servlet->writeContentOutput() was called with the unchanged ShippingMethod'
    );
}

sub test_ChangePortalShippingMethod_fail_no_PortalConfig {
    my $self = shift;
    my $SubTestName = ( caller(0) )[3];


    my $MockedShop = $self->_createShop({  });

    my $MockedPortalConfig = $self->_createPortalConfig({
         MockedShop => $MockedShop,
    });

    my $hFormValues = {};
    my $MockedServletForm  = $self->_createServletForm($hFormValues);
    my $MockedServletResponse = $self->_createServletResponse();
    my $MockedServlet = $self->_createServlet($MockedPortalConfig, $MockedServletForm, $MockedServletResponse);

    my $SUT = $self->_createSUT();

    $SUT->ChangePortalShippingMethod($MockedServlet);

    my $Code = GetParametersFromMockifyCall($MockedServletResponse, 'code')->[0];
    my $aParams = GetParametersFromMockifyCall($MockedServlet, 'params');
    my $hContent = GetParametersFromMockifyCall($MockedServlet, 'writeContentOutput')->[0];
    # actual tests
    is(
        $Code,
        HTTP_OK,
        $SubTestName . ' - Ensure Ensure HTTP response code was HTTP_OK'
    );
    is_deeply(
        $aParams,
        [ 'ContentType' => 'application/json' ],
        $SubTestName . ' - Ensure ContentType of application/json'
    );
    is_deeply(
        $hContent->{'ShippingMethodID'},
        undef,
        $SubTestName . ' - Ensure $Servlet->writeContentOutput() was called without ShippingMethod, because there was no PortalConfig to read from'
    );
}

sub test_UploadAllProducts {
    my $self = shift;
    my $SubTestName = ( caller(0) )[3];


    my $MockedShop = $self->_createShop({  });

    my $MockedPortalConfig = $self->_createPortalConfig({
        MockedShop => $MockedShop,
    });
    my $MockedServlet = $self->_createServlet($MockedPortalConfig);

    my $SUT = $self->_createSUT();

    $SUT->UploadAllProducts($MockedServlet);

    my $hParams = GetParametersFromMockifyCall($MockedPortalConfig, 'set')->[0];

    # actual tests
    is_deeply(
        $hParams,
            { 'ExportAllProducts' => 1 },
        $SubTestName . ' - Ensure that the ExportAllProducts boolean was set to true'
    );
}

#----------------------------------------------------------------------------------------
# Mockery
#----------------------------------------------------------------------------------------
sub _createSUT {
    my $self = shift;
    my ($hParams ) = @_;

    my $MockifySut = Test::Mockify::Sut->new('DE_EPAGES::GoogleSmartShopping::UI::ProductFeed');

    $MockifySut->mockImported('DE_EPAGES::GoogleSmartShopping::API::Tools', 'LogGoogleSmartShopping')
        ->whenAny()
        ->thenReturnUndef();

    if (defined $hParams->{'GoogleApiService'}) {
        $MockifySut->mockStatic('DE_EPAGES::GoogleSmartShopping::API::GoogleApiService::new')
            ->when(String(), HashRef())
            ->thenReturn($hParams->{'GoogleApiService'});
    }

    if (defined($hParams->{'NewGooglePortalService'})) {
        $MockifySut->mock('_createPortalConfig')
            ->whenAny()
            ->thenReturn($hParams->{'NewGooglePortalService'});
    }
    else {
        $MockifySut->mock('_createPortalConfig')
            ->whenAny()
            ->thenReturnUndef();
    }

    $MockifySut->mock('_viewPagedChildren')
        ->whenAny()
        ->thenReturnUndef();

    $MockifySut->spy('_areProductUploadRequirementsMet')->when(Object());

    return $MockifySut->getMockObject();
}

sub _createPortalConfig {
    my $self = shift;
    my ($hParams) = @_;

    my $hAttributes = {
        'ShippingMethod' => $self->_createShippingMethod(),
        'NameOrAlias' => 'NameOrAlias' ,
        'PortalConfigClass' => 'GoogleSmartShopping',
        'ID' => '12345',
        'Shop' => $hParams->{'MockedShop'},
        'Products' => $hParams->{'Products'} // [],
    };

    my $MockifyPortal = DE_EPAGES::Test::Mock::MockifyEpagesObject->new(
        'DE_EPAGES::GoogleSmartShopping::API::Object::GoogleSmartShoppingPortalConfig', $hAttributes
    );
    $MockifyPortal->addMockWithReturnValueAndParameterCheck('set', undef, [ 'hashref' ]);

    return $MockifyPortal->getMockObject();
}

sub _createShop {
    my $self = shift;
    my ($hParams) = @_;

    my $hAttributes = {
        'Address'                   => $hParams->{'MockedAddress'} // undef,
        'GoogleSmartShoppingConfig' => $hParams->{'MockedGoogleConfig'} // undef,
    };
    my $MockifyShop = DE_EPAGES::Test::Mock::MockifyEpagesObject->new('DE_EPAGES::Shop::API::Object::Shop', $hAttributes);

    $MockifyShop->addMockWithReturnValueAndParameterCheck('existsChild', defined($hParams->{'MockedGoogleConfig'}), [{'string'=>'GoogleSmartShoppingConfig'}]);
    $MockifyShop->addMockWithReturnValueAndParameterCheck('loadObject',$self->_createShippingMethod(), ['string', 'string']);


    return $MockifyShop->getMockObject();
}

sub _createGoogleConfig {
    my $self = shift;
    my ($hConfig) = @_;

    $hConfig->{'AreGoogleMerchantCenterRequirementsMet'} //= 0;

    my $hAttributes = {
        ChecklistBillingTOC                => $hConfig->{'AreGoogleMerchantCenterRequirementsMet'},
        ChecklistContactInfo               => $hConfig->{'AreGoogleMerchantCenterRequirementsMet'},
        ChecklistGoogleAdsPolicies         => $hConfig->{'AreGoogleMerchantCenterRequirementsMet'},
        ChecklistCookieNotice              => $hConfig->{'AreGoogleMerchantCenterRequirementsMet'},
        ChecklistReturnPolicy              => $hConfig->{'AreGoogleMerchantCenterRequirementsMet'},
        MerchantCenterAccountID            => $hConfig->{'MerchantCenterAccountID'} // undef,
        GoogleAdsAccountID                 => $hConfig->{'GoogleAdsAccountID'} // undef,
        AdsCampaignId                      => $hConfig->{'AdsCampaignId'} // undef,
        MerchantCenterMultiClientAccountID => $hConfig->{'MerchantCenterMultiClientAccountID'} // undef,
        AreMerchantCenterAndAdsLinked      => $hConfig->{'AreMerchantCenterAndAdsLinked'} // 1,
    };
    my $Mockify = DE_EPAGES::Test::Mock::MockifyEpagesObject->new(
        'DE_EPAGES::GoogleSmartShopping::API::Object::GoogleSmartShoppingConfig', $hAttributes
    );

    return $Mockify->getMockObject();
}

sub _createServlet {
    my $self = shift;
    my ($PortalConfig, $ServletForm, $ServletResponse) = @_;

    my $MockifyServlet = DE_EPAGES::Test::Mock::MockifyServlet->new($PortalConfig);
    if (defined $ServletForm && defined $ServletResponse) {
        $MockifyServlet->addMockWithReturnValue('form', $ServletForm);
        $MockifyServlet->addMockWithReturnValue('response', $ServletResponse);
        $MockifyServlet->addMockWithReturnValueAndParameterCheck('writeContentOutput', undef, [ 'hashref' ]);
        $MockifyServlet->addMockWithReturnValueAndParameterCheck('params', undef, [ 'string', 'string' ]);
    }


    return $MockifyServlet->getMockObject();
}

sub _createServletForm {
    my $self = shift;
    my ($hPostData) = @_;

    my $Mockify = Test::Mockify->new('DE_EPAGES::Presentation::API::Form');

    $Mockify->mock('formValues')
        ->when()
        ->thenReturn($hPostData);

    $Mockify->mock('executeFormError')
        ->when(HashRef())
        ->thenThrowError('Something bad happened! fail;<');

    return $Mockify->getMockObject();
}

sub _createServletResponse {
    my $self = shift;

    my $Mockify = Test::Mockify->new('HTTP::Response');
    $Mockify->mock('code')
        ->whenAny()
        ->thenReturnUndef();

    return $Mockify->getMockObject();
}

sub _createAddress {
    my $self = shift;
    my ($hCountry) = @_;

    my $hParameterList = {
        'Country'   => $hCountry ,
    };

    my $MockAddress = DE_EPAGES::Test::Mock::MockifyEpagesObject->new(
        'DE_EPAGES::Address::API::Object::Address',
        $hParameterList
    );

    return $MockAddress->getMockObject();
}

sub _createGoogleApiService {
    my $self = shift;
    my ($hServiceResponse) = @_;

    my $Mockify = Test::Mockify->new('DE_EPAGES::GoogleSmartShopping::API::GoogleApiService');

    $Mockify->mock('getRatedProductsStatistics')
        ->when()
        ->thenReturn($hServiceResponse->{'RatedProductsStatistics'} // {}
    );
    return $Mockify->getMockObject();
}

sub _createShippingMethod {
    my $self = shift;
    my $hParameterList = {
        'ID'          => DEFAULT_SHIPPING_METHOD_ID,
        'NameOrAlias' => DEFAULT_SHIPPING_METHOD_ALIAS
    };
    my $MockAddress = DE_EPAGES::Test::Mock::MockifyEpagesObject->new(
        'DE_EPAGES::ShippingTypes::API::Object::ShippingMethodWeightOfBasket',
        $hParameterList
    );

    return $MockAddress->getMockObject();
}

__PACKAGE__->runScript();
