use strict;

use Getopt::Long;
use DE_EPAGES::Core::API::Script qw( RunScript );
use DE_EPAGES::Core::API::Error qw(ExistsError GetError );
use DE_EPAGES::Core::API::Log qw(LogError );
use DE_EPAGES::Core::API::Warning qw (Warning);
use DE_EPAGES::Database::API::Connection qw( RunOnStore GetCurrentDBHandle Transaction );
use DE_EPAGES::Object::API::Factory qw( LoadRootObject LoadClassByAlias ResetFactoryCaches);
use DE_EPAGES::RemoteSearch::API::RemoteSearchImpl qw (GetOrCreateSearchIndex );
use DE_EPAGES::WebInterface::API::MessageCenter qw( SynchronizeCache );
use DE_EPAGES::Tax::API::Shop qw( BackupShopTaxMatrix );

RunScript( Sub => \&Main );

sub Main {
    my ($Help, $StoreName, $Password, $ShopAlias, $TestMode) ;
    GetOptions(
        'help'                  => \$Help,
        'storename=s'           => \$StoreName,
        'password=s'            => \$Password,
        'shop=s'                => \$ShopAlias,
        'test'                  => \$TestMode
    );
    usage() if $Help;

    unless ($StoreName) {
        print STDERR "Missing parameter storename\n\n";
        usage();
    }
    unless ($ShopAlias) {
        print STDERR "Missing parameter shop\n\n";
        usage();
    }
    RunOnStore(
        Store    => $StoreName,
        Password => $Password,
        Sub      => sub {

            my $time1 = time();

            my $Shop = LoadRootObject->folder('Shops')->child($ShopAlias);
            printf "================================================================================ \n";
            eval {
                #close shop:
                $Shop->set({'IsClosed'=>1}) unless $TestMode;
                Transaction(sub {
                    _resetCustomizedShopTaxMatrix($StoreName, $Shop, $TestMode);
                    die "$StoreName rollback everything because it is a test\n" if $TestMode;
                });
            };
            if (ExistsError()) {
                my $Error = GetError();
                LogError($Error);
                printf "$StoreName There was an issue with shop %s, changes have been rolled back.\n", $Shop->alias;
                printf "$StoreName The reported error was '%s' (Details in error log)\n", $Error->shortMessage;
                $Shop->set({'IsClosed'=>0}) unless $TestMode;
            } else {
                unless ($TestMode) {
                    #update search index, sync cache
                    eval {
                        GetOrCreateSearchIndex($Shop)->initialize();
                        SynchronizeCache();
                    };
                    Warning(GetError()) if ExistsError();
                    printf "$StoreName Update Cache, update SearchIndex for shop %s.\n", $Shop->alias;
                } else {
                    printf "$StoreName TestMode: Update Cache, update SearchIndex for shop %s.\n", $Shop->alias;
                }
            }
            ResetFactoryCaches() unless $TestMode;
            printf "================================================================================ \n";
            my $time2   = time;
            my $seconds = $time2 - $time1;
            printf "\n$StoreName need %.3f seconds over all\n", $seconds;
        }
    );

    eval { SynchronizeCache(); };
    Warning( GetError() ) if ExistsError();
    return;

}

sub _resetCustomizedShopTaxMatrix {
    my ( $StoreName, $Shop, $TestMode ) = @_;

    my $ShopAlias = $Shop->alias();
    print "$StoreName _resetCustomizedShopTaxMatrix(Shop:$ShopAlias, TestMode:$TestMode)\n";
    my $ShopTaxMatrix = $Shop->get('TaxMatrix');
    my $ShopTaxMatrixID =$ShopTaxMatrix->id;

    # Run script only for shops with customized TaxMatrix
    my $TaxMatrixAlias = $Shop->get('TaxMatrix')->alias;
    my $SystemTaxMatrix = LoadRootObject()->child($TaxMatrixAlias);
    my $SystemTaxMatrixID =$SystemTaxMatrix->id;
    if ($ShopTaxMatrixID == $SystemTaxMatrixID) {
        printf "$StoreName Skip " .$ShopAlias. "No customized TaxMatrix.\n";
        $Shop->set({'IsClosed'=>0});
    } else {
        unless ($TestMode) {
            printf "Backing up TaxMatrix %s (%s)\n",$ShopTaxMatrix->alias, $ShopTaxMatrix->id;
            BackupShopTaxMatrix($ShopTaxMatrix,0);
        }
        # update taxmatrix via sql except orders (lineitemcontainers, lineitems)
        _updateCustomerTaxAreas($StoreName, $Shop,$ShopTaxMatrixID,$SystemTaxMatrixID);
        _updateProducts($StoreName, $Shop,$ShopTaxMatrixID,$SystemTaxMatrixID);
        _updatePaymentMethods($StoreName, $Shop,$ShopTaxMatrixID,$SystemTaxMatrixID);
        _updateShippingMethods($StoreName, $Shop,$ShopTaxMatrixID,$SystemTaxMatrixID);
        _updateShippingOptions($StoreName, $Shop,$ShopTaxMatrixID,$SystemTaxMatrixID);
        _updateTaxMatrixReference($StoreName, $Shop,$SystemTaxMatrixID);
        $Shop->set({'IsClosed'=>0});
        print "$StoreName finish _resetCustomizedShopTaxMatrix(Shop:$ShopAlias, TestMode:$TestMode)\n";

    }
    return;
}

sub _updateCustomerTaxAreas {
    my ($StoreName,$Shop,$ShopTaxMatrixID,$SystemTaxMatrixID) = @_;

    my $ShopAlias =   $Shop->alias;
    my @Parameter = ($ShopTaxMatrixID,$SystemTaxMatrixID );
    my $dbi= GetCurrentDBHandle();
    print "$StoreName start _updateCustomerTaxAreas ($ShopAlias,$ShopTaxMatrixID,$SystemTaxMatrixID)\n";

    my $sth = $dbi->executeStatement(<<END_SQL, \@Parameter);
UPDATE customer c
    INNER JOIN taxarea shopta   ON shopta.taxareaid = c.taxareaid
    INNER JOIN taxmatrix shoptm ON shoptm.taxmatrixid = shopta.taxmatrixid
    INNER JOIN object shoptao   ON shoptao.objectid = shopta.taxareaid
    INNER JOIN object systao    ON systao.alias = shoptao.alias
    INNER JOIN taxarea systa    ON systa.taxareaid = systao.objectid
    SET c.taxareaid = systa.taxareaid
    WHERE shoptm.taxmatrixid = ?
    AND systa.taxmatrixid = ?;
END_SQL
    printf "$StoreName Affected rows (_updateCustomerTaxAreas) %s\n", $sth->rows();
    return;
}

sub _updateProducts {
    my ($StoreName,$Shop,$ShopTaxMatrixID,$SystemTaxMatrixID) = @_;

    my $ShopAlias =   $Shop->alias;
    my @Parameter = ($ShopTaxMatrixID,$SystemTaxMatrixID );
    my $dbi= GetCurrentDBHandle();
    print "$StoreName start _updateProducts ($ShopAlias,$ShopTaxMatrixID,$SystemTaxMatrixID)\n";

    my $sth = $dbi->executeStatement(<<END_SQL, \@Parameter);
UPDATE product p
    INNER JOIN taxclass shoptc ON shoptc.taxclassid = p.taxclassid
    INNER JOIN taxmatrix shoptm ON shoptm.taxmatrixid = shoptc.taxmatrixid
    INNER JOIN object shoptco  ON shoptco.objectid = shoptc.taxclassid
    INNER JOIN object systco   ON systco.alias = shoptco.alias
    INNER JOIN taxclass systc ON systc.taxclassid = systco.objectid
SET p.taxclassid = systc.taxclassid
    WHERE shoptm.taxmatrixid = ?
    AND systc.taxmatrixid = ?;
END_SQL
    printf "$StoreName Affected rows (_updateProducts) %s\n", $sth->rows();
    return;
}

sub _updatePaymentMethods {
    my ($StoreName,$Shop,$ShopTaxMatrixID,$SystemTaxMatrixID) = @_;

    my $ShopAlias =   $Shop->alias;
    my @Parameter = ($ShopTaxMatrixID,$SystemTaxMatrixID );
    my $dbi= GetCurrentDBHandle();
    print "$StoreName start _updatePaymentMethods ($ShopAlias,$ShopTaxMatrixID,$SystemTaxMatrixID)\n";

    my $sth = $dbi->executeStatement(<<END_SQL, \@Parameter);
UPDATE paymentmethod pm
    INNER JOIN taxclass shoptc ON shoptc.taxclassid = pm.taxclassid
    INNER JOIN taxmatrix shoptm ON shoptm.taxmatrixid = shoptc.taxmatrixid
    INNER JOIN object shoptco  ON shoptco.objectid = shoptc.taxclassid
    INNER JOIN object systco   ON systco.alias = shoptco.alias
    INNER JOIN taxclass systc ON systc.taxclassid = systco.objectid
SET pm.taxclassid = systc.taxclassid
    WHERE shoptm.taxmatrixid = ?
    AND systc.taxmatrixid = ?;
END_SQL
    printf "$StoreName Affected rows (_updatePaymentMethods) %s\n", $sth->rows();
    return;
}

sub _updateShippingMethods {
    my ($StoreName,$Shop,$ShopTaxMatrixID,$SystemTaxMatrixID) = @_;

    my $ShopAlias =   $Shop->alias;
    my @Parameter = ($ShopTaxMatrixID,$SystemTaxMatrixID );
    my $dbi= GetCurrentDBHandle();
    print "$StoreName start _updateShippingMethods ($ShopAlias,$ShopTaxMatrixID,$SystemTaxMatrixID)\n";

    my $sth = $dbi->executeStatement(<<END_SQL, \@Parameter);
UPDATE shippingmethod sm
    INNER JOIN taxclass shoptc ON shoptc.taxclassid = sm.taxclassid
    INNER JOIN taxmatrix shoptm ON shoptm.taxmatrixid = shoptc.taxmatrixid
    INNER JOIN object shoptco  ON shoptco.objectid = shoptc.taxclassid
    INNER JOIN object systco   ON systco.alias = shoptco.alias
    INNER JOIN taxclass systc ON systc.taxclassid = systco.objectid
SET sm.taxclassid = systc.taxclassid
    WHERE shoptm.taxmatrixid = ?
    AND systc.taxmatrixid = ?;
END_SQL
    printf "$StoreName Affected rows (_updateShippingMethods) %s\n", $sth->rows();
    return;
}

sub _updateShippingOptions {
    my ($StoreName,$Shop,$ShopTaxMatrixID,$SystemTaxMatrixID) = @_;

    my $ShopAlias =   $Shop->alias;
    my @Parameter = ($ShopTaxMatrixID,$SystemTaxMatrixID );
    my $dbi= GetCurrentDBHandle();
    print "$StoreName start _updateShippingMethods ($ShopAlias,$ShopTaxMatrixID,$SystemTaxMatrixID)\n";

    my $sth = $dbi->executeStatement(<<END_SQL, \@Parameter);
UPDATE shippingoption so
    INNER JOIN taxclass shoptc ON shoptc.taxclassid = so.taxclassid
    INNER JOIN taxmatrix shoptm ON shoptm.taxmatrixid = shoptc.taxmatrixid
    INNER JOIN object shoptco  ON shoptco.objectid = shoptc.taxclassid
    INNER JOIN object systco   ON systco.alias = shoptco.alias
    INNER JOIN taxclass systc ON systc.taxclassid = systco.objectid
SET so.taxclassid = systc.taxclassid
    WHERE shoptm.taxmatrixid = ?
    AND systc.taxmatrixid = ?;
END_SQL
    printf "$StoreName Affected rows (_updateShippingOptions) %s\n", $sth->rows();
    return;
}

sub _updateTaxMatrixReference {
    my ($StoreName,$Shop,$SystemTaxMatrixID) = @_;

    my $ShopAlias =   $Shop->alias;

    my $ShopTaxMatrixAttributeID = LoadClassByAlias('Shop')->attribute('TaxMatrix')->id;
    my @Parameter = ($SystemTaxMatrixID,$Shop->id, $ShopTaxMatrixAttributeID );
    my $dbi= GetCurrentDBHandle();
    print "$StoreName start _updateTaxMatrixReference ($ShopAlias,$ShopTaxMatrixAttributeID,$SystemTaxMatrixID)\n";

    my $sth = $dbi->executeStatement(<<END_SQL, \@Parameter);
UPDATE referenceattribute
    SET referencedobjectid = ?
WHERE objectid = ?
    AND attributeid = ?;
END_SQL

    printf "$StoreName Affected rows (_updateTaxMatrixReference) %s\n", $sth->rows();
    return;
}


sub usage {
    print <<END_USAGE;
Description:
    Resets Shop TaxMatrix to default System TaxMatrix
Usage:
    \$PERL $0 [options] [flags]
Options:
    -storename      store name for the database connection                              required
    -shop           name of the shop                                                    optional
flags:
    -test               Run script in test mode                                         optional
Example:
    \$PERL $0 -storename Store -shop DemoShop
    \$PERL $0 -storename Store -shop DemoShop -test
END_USAGE
    exit(0);
}
